@@ -308,10 +308,8 @@ class Config(OrderedDict):
             
         copy()
             Returns a blank copy of the current config, with a new recorder.
-    
-        
     """
-        
+    
     def __init__(self, *args, config_name : str = "config", **kwargs):
         """
         See help(Config) for a description of this class.
@@ -906,8 +904,13 @@ class Config(OrderedDict):
         for c in self._children:
             if c[:1] == "_":
                 continue
-            inputs[c] = self._children[c].unique_id() 
-        return uniqueHashExt(length=length,parse_functions=parse_functions)(inputs)
+            # collect ID for the child
+            child_data = self._children[c].unique_id() 
+            # we only register children if they have keys.
+            # this way we do not trigger a change in ID simply due to a failed read access.
+            if child_data != "":
+                inputs[c]  = child_data
+        return uniqueHashExt(length=length,parse_functions=parse_functions)(inputs) if len(inputs) > 0 else ""
 
     # magic
     # -----
